@@ -6,7 +6,7 @@
  * Copyright (C) 2002-2011
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: eventhandler.c 5859 2011-07-09 11:28:20Z rousseau $
+ * $Id: eventhandler.c 6391 2012-07-26 18:44:54Z rousseau $
  */
 
 /**
@@ -170,7 +170,7 @@ LONG EHDestroyEventHandler(READER_CONTEXT * rContext)
 
 		if ((IFD_SUCCESS == rv) && (dwGetSize == sizeof(fct)))
 		{
-			Log1(PCSC_LOG_INFO, "Request stoping of polling thread");
+			Log1(PCSC_LOG_INFO, "Request stopping of polling thread");
 			fct(rContext->slot);
 		}
 		else
