@@ -10,7 +10,7 @@
  * Copyright (C) 2009
  *  Jean-Luc Giraud <jlgiraud@googlemail.com>
  *
- * $Id: readerfactory.c 6356 2012-06-25 13:38:46Z rousseau $
+ * $Id: readerfactory.c 6397 2012-08-02 14:23:26Z rousseau $
  */
 
 /**
@@ -58,6 +58,7 @@ static int ConfigFileCRC = 0;
 static pthread_mutex_t LockMutex = PTHREAD_MUTEX_INITIALIZER;
 
 #define IDENTITY_SHIFT 16
+static LONG removeReader(READER_CONTEXT * sReader);
 
 static int RDR_CLIHANDLES_seeker(const void *el, const void *key)
 {
@@ -78,6 +79,33 @@ static int RDR_CLIHANDLES_seeker(const void *el, const void *key)
 }
 
 
+LONG _RefReader(READER_CONTEXT * sReader)
+{
+	if (0 == sReader->reference)
+		return SCARD_E_READER_UNAVAILABLE;
+
+	pthread_mutex_lock(&sReader->reference_lock);
+	sReader->reference += 1;
+	pthread_mutex_unlock(&sReader->reference_lock);
+
+	return SCARD_S_SUCCESS;
+}
+
+LONG _UnrefReader(READER_CONTEXT * sReader)
+{
+	if (0 == sReader->reference)
+		return SCARD_E_READER_UNAVAILABLE;
+
+	pthread_mutex_lock(&sReader->reference_lock);
+	sReader->reference -= 1;
+	pthread_mutex_unlock(&sReader->reference_lock);
+
+	if (0 == sReader->reference)
+		removeReader(sReader);
+
+	return SCARD_S_SUCCESS;
+}
+
 LONG RFAllocateReaderSpace(unsigned int customMaxReaderHandles)
 {
 	int i;	/* Counter */
@@ -120,7 +148,7 @@ LONG RFAddReader(const char *readerNameLong, int port, const char *library,
 		return SCARD_E_INVALID_VALUE;
 
 	/* allocate memory that is automatically freed */
-	readerName = alloca(strlen(readerNameLong));
+	readerName = alloca(strlen(readerNameLong)+1);
 	strcpy(readerName, readerNameLong);
 
 	/* Reader name too long? also count " 00 00"*/
@@ -218,6 +246,11 @@ LONG RFAddReader(const char *readerNameLong, int port, const char *library,
 		NULL);
 	sReadersContexts[dwContext]->powerState = POWER_STATE_UNPOWERED;
 
+	/* reference count */
+	(void)pthread_mutex_init(&sReadersContexts[dwContext]->reference_lock,
+		NULL);
+	sReadersContexts[dwContext]->reference = 1;
+
 	/* If a clone to this reader exists take some values from that clone */
 	if (parentNode >= 0 && parentNode < PCSCLITE_MAX_READERS_CONTEXTS)
 	{
@@ -411,6 +444,11 @@ LONG RFAddReader(const char *readerNameLong, int port, const char *library,
 			NULL);
 		sReadersContexts[dwContextB]->powerState = POWER_STATE_UNPOWERED;
 
+		/* reference count */
+		(void)pthread_mutex_init(&sReadersContexts[dwContextB]->reference_lock,
+			NULL);
+		sReadersContexts[dwContextB]->reference = 1;
+
 		/* Call on the parent driver to see if the slots are thread safe */
 		dwGetSize = sizeof(ucThread);
 		rv = IFDGetCapabilities((sReadersContexts[dwContext]),
@@ -468,14 +506,38 @@ LONG RFAddReader(const char *readerNameLong, int port, const char *library,
 
 LONG RFRemoveReader(const char *readerName, int port)
 {
-	LONG rv;
-	READER_CONTEXT * sContext;
+	char lpcStripReader[MAX_READERNAME];
+	int i;
 
 	if (readerName == NULL)
 		return SCARD_E_INVALID_VALUE;
 
-	while (SCARD_S_SUCCESS ==
-		RFReaderInfoNamePort(port, readerName, &sContext))
+	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
+	{
+		if (sReadersContexts[i]->vHandle != 0)
+		{
+			strncpy(lpcStripReader,
+				sReadersContexts[i]->readerState->readerName,
+				sizeof(lpcStripReader));
+			lpcStripReader[strlen(lpcStripReader) - 6] = 0;
+
+			/* Compare only the significant part of the reader name */
+			if ((strncmp(readerName, lpcStripReader, MAX_READERNAME - sizeof(" 00 00")) == 0)
+				&& (port == sReadersContexts[i]->port))
+			{
+				/* remove the reader */
+				UNREF_READER(sReadersContexts[i])
+			}
+		}
+	}
+
+	return SCARD_S_SUCCESS;
+}
+
+LONG removeReader(READER_CONTEXT * sContext)
+{
+	LONG rv;
+
 	{
 		/* Try to destroy the thread */
 		if (sContext -> pthThread)
@@ -673,6 +735,9 @@ LONG RFReaderInfo(const char *readerName, READER_CONTEXT ** sReader)
 			if (strcmp(readerName,
 				sReadersContexts[i]->readerState->readerName) == 0)
 			{
+				/* Increase reference count */
+				REF_READER(sReadersContexts[i])
+
 				*sReader = sReadersContexts[i];
 				return SCARD_S_SUCCESS;
 			}
@@ -682,37 +747,6 @@ LONG RFReaderInfo(const char *readerName, READER_CONTEXT ** sReader)
 	return SCARD_E_UNKNOWN_READER;
 }
 
-LONG RFReaderInfoNamePort(int port, const char *readerName,
-	READER_CONTEXT * * sReader)
-{
-	char lpcStripReader[MAX_READERNAME];
-	int i;
-
-	for (i = 0; i < PCSCLITE_MAX_READERS_CONTEXTS; i++)
-	{
-		if (sReadersContexts[i]->vHandle != 0)
-		{
-			int tmplen;
-
-			strncpy(lpcStripReader,
-				sReadersContexts[i]->readerState->readerName,
-				sizeof(lpcStripReader));
-			tmplen = strlen(lpcStripReader);
-			lpcStripReader[tmplen - 6] = 0;
-
-			/* Compare only the significant part of the reader name */
-			if ((strncmp(readerName, lpcStripReader, MAX_READERNAME - sizeof(" 00 00")) == 0)
-				&& (port == sReadersContexts[i]->port))
-			{
-				*sReader = sReadersContexts[i];
-				return SCARD_S_SUCCESS;
-			}
-		}
-	}
-
-	return SCARD_E_INVALID_VALUE;
-}
-
 LONG RFReaderInfoById(SCARDHANDLE hCard, READER_CONTEXT * * sReader)
 {
 	int i;
@@ -728,6 +762,9 @@ LONG RFReaderInfoById(SCARDHANDLE hCard, READER_CONTEXT * * sReader)
 			(void)pthread_mutex_unlock(&sReadersContexts[i]->handlesList_lock);
 			if (currentHandle != NULL)
 			{
+				/* Increase reference count */
+				REF_READER(sReadersContexts[i])
+
 				*sReader = sReadersContexts[i];
 				return SCARD_S_SUCCESS;
 			}
